// Generated by CoffeeScript 1.3.3
(function() {
  var async, destroyFile, exec, fs, fs_utils, generateFile, generateOrDestroy, helpers, install, isCompilerFor, logger, mkdirp, ncp, spawn, sysPath, watchApplication, _ref;

  async = require('async');

  _ref = require('child_process'), exec = _ref.exec, spawn = _ref.spawn;

  fs = require('fs');

  mkdirp = require('mkdirp');

  ncp = require('ncp').ncp;

  sysPath = require('path');

  helpers = require('./helpers');

  logger = require('./logger');

  fs_utils = require('./fs_utils');

  isCompilerFor = function(path) {
    return function(plugin) {
      var pattern;
      pattern = plugin.pattern ? plugin.pattern : plugin.extension ? RegExp("\\." + plugin.extension + "$") : null;
      return (typeof plugin.compile === 'function') && !!(pattern != null ? pattern.test(path) : void 0);
    };
  };

  watchApplication = function(persistent, rootPath, config, callback) {
    var assetPath, directories, fileList, ignored, _base, _ref1, _ref2, _ref3;
    if ((_ref1 = config.buildPath) == null) {
      config.buildPath = sysPath.join(rootPath, 'public');
    }
    if ((_ref2 = config.server) == null) {
      config.server = {};
    }
    if ((_ref3 = (_base = config.server).port) == null) {
      _base.port = 3333;
    }
    config.rootPath = rootPath;
    assetPath = sysPath.join(rootPath, 'app', 'assets');
    ignored = function(path) {
      return helpers.startsWith(path, assetPath) || helpers.startsWith(sysPath.basename(path), '_');
    };
    if (persistent && config.server.run) {
      helpers.startServer(config.server.port, config.buildPath, config);
    }
    directories = ['app', 'vendor'].map(function(dir) {
      return sysPath.join(rootPath, dir);
    });
    fileList = new fs_utils.SourceFileList;
    return helpers.loadPlugins(config, function(error, plugins) {
      var addToFileList, removeFromFileList, start, valid, watcher, writer;
      if (error != null) {
        return logger.error(error);
      }
      start = null;
      addToFileList = function(isPluginHelper) {
        return function(path) {
          var compiler;
          start = Date.now();
          logger.log('debug', "File '" + path + "' was changed");
          if (ignored(path)) {
            return fileList.resetTimer();
          }
          compiler = plugins.filter(isCompilerFor(path))[0];
          if (!compiler) {
            return;
          }
          return fileList.add({
            path: path,
            compiler: compiler,
            isPluginHelper: isPluginHelper
          });
        };
      };
      removeFromFileList = function(path) {
        if (ignored(path)) {
          return fileList.resetTimer();
        }
        return fileList.remove(path);
      };
      plugins.forEach(function(plugin) {
        var includePathes;
        if (plugin.include == null) {
          return;
        }
        includePathes = typeof plugin.include === 'function' ? plugin.include() : plugin.include;
        return includePathes.forEach(addToFileList(true));
      });
      writer = new fs_utils.FileWriter(config, plugins);
      watcher = (new fs_utils.FileWatcher).add(directories).on('change', addToFileList(false)).on('remove', removeFromFileList);
      fileList.on('resetTimer', function() {
        return writer.write(fileList);
      });
      valid = function(path) {
        return !watcher.invalid.test(sysPath.basename(path));
      };
      writer.on('write', function(result) {
        assetPath = sysPath.join(rootPath, 'app', 'assets');
        return ncp(assetPath, config.buildPath, {
          filter: valid
        }, function(error) {
          if (error != null) {
            logger.error("Asset compilation failed: " + error);
          }
          logger.info("compiled in " + (Date.now() - start) + "ms.");
          if (!persistent) {
            watcher.close();
          }
          return callback(null, result);
        });
      });
      return watcher;
    });
  };

  generateFile = function(path, data, callback) {
    var parentDir, write;
    parentDir = sysPath.dirname(path);
    write = function() {
      logger.info("create " + path);
      return fs.writeFile(path, data, callback);
    };
    return sysPath.exists(parentDir, function(exists) {
      if (exists) {
        return write();
      }
      logger.info("create " + parentDir);
      return mkdirp(parentDir, parseInt(755, 8), function(error) {
        if (error != null) {
          return logger.error;
        }
        return write();
      });
    });
  };

  destroyFile = function(path, callback) {
    return fs.unlink(path, function(error) {
      if (error != null) {
        return logger.error("" + error);
      }
      logger.info("destroy " + path);
      return callback(error);
    });
  };

  generateOrDestroy = function(generate, options, callback) {
    var appPath, config, configSection, extension, generateOrDestroyFile, initFile, initTests, languageType, name, parentDir, rootPath, testPath, type, _ref1;
    rootPath = options.rootPath, type = options.type, name = options.name, config = options.config, parentDir = options.parentDir;
    generateOrDestroyFile = generate ? generateFile : destroyFile;
    appPath = sysPath.join(rootPath, 'app');
    testPath = sysPath.join(rootPath, 'test', 'unit');
    languageType = (function() {
      switch (type) {
        case 'collection':
        case 'model':
        case 'router':
        case 'view':
          return 'javascript';
        case 'style':
          return 'stylesheet';
        default:
          return type;
      }
    })();
    configSection = config.files[helpers.pluralize(languageType)];
    extension = (function() {
      if ((_ref1 = configSection != null ? configSection.defaultExtension : void 0) != null) {
        return _ref1;
      } else {
        switch (languageType) {
          case 'javascript':
            return 'coffee';
          case 'stylesheet':
            return 'styl';
          case 'template':
            return 'eco';
          default:
            return '';
        }
      }
    })();
    if (type === 'router' || type === 'view') {
      name += "_" + type;
    }
    if (parentDir == null) {
      parentDir = languageType === 'template' ? sysPath.join('views', "" + (helpers.pluralize(type))) : "" + (helpers.pluralize(type));
    }
    logger.log('debug', "Initializing file of type '" + languageType + "' with extension '" + extension + "'");
    initFile = function(parentDir, callback) {
      var fullPath;
      fullPath = sysPath.join(appPath, parentDir, "" + name + "." + extension);
      return helpers.loadPlugins(config, function(error, plugins) {
        var data, generator, plugin, _ref2, _ref3;
        plugin = plugins.filter(function(plugin) {
          return plugin.extension === extension;
        })[0];
        generator = plugin != null ? (_ref2 = plugin.generators) != null ? (_ref3 = _ref2[config.framework || 'backbone']) != null ? _ref3[type] : void 0 : void 0 : void 0;
        data = generator != null ? typeof generator === 'function' ? generator(name) : generator : '';
        if (generate) {
          return generateFile(fullPath, data, callback);
        } else {
          return destroyFile(fullPath, callback);
        }
      });
    };
    initTests = function(parentDir, callback) {
      var fullPath;
      if (languageType !== 'javascript') {
        return callback();
      }
      fullPath = sysPath.join(testPath, parentDir, "" + name + "." + extension);
      if (generate) {
        return generateFile(fullPath, '', callback);
      } else {
        return destroyFile(fullPath, callback);
      }
    };
    return initFile(parentDir, function() {
      return initTests(parentDir, function() {
        return callback();
      });
    });
  };

  install = function(rootPath, callback) {
    var prevDir;
    if (callback == null) {
      callback = (function() {});
    }
    prevDir = process.cwd();
    process.chdir(rootPath);
    logger.info('Installing packages...');
    return exec('npm install', function(error, stdout, stderr) {
      process.chdir(prevDir);
      return callback(stderr, stdout);
    });
  };

  exports["new"] = function(options, callback) {
    var buildPath, rootPath, template;
    if (callback == null) {
      callback = (function() {});
    }
    rootPath = options.rootPath, buildPath = options.buildPath, template = options.template;
    if (buildPath == null) {
      buildPath = sysPath.join(rootPath, 'build');
    }
    if (template == null) {
      template = sysPath.join(__dirname, '..', 'template', 'base');
    }
    return sysPath.exists(rootPath, function(exists) {
      if (exists) {
        return logger.error("Directory '" + rootPath + "' already exists");
      }
      return mkdirp(rootPath, parseInt(755, 8), function(error) {
        if (error != null) {
          return logger.error(error);
        }
        return ncp(template, rootPath, function(error) {
          if (error != null) {
            return logger.error(error);
          }
          logger.info('Created brunch directory layout');
          return install(rootPath, callback);
        });
      });
    });
  };

  exports.build = function(rootPath, config, callback) {
    if (callback == null) {
      callback = (function() {});
    }
    return watchApplication(false, rootPath, config, callback);
  };

  exports.watch = function(rootPath, config, callback) {
    if (callback == null) {
      callback = (function() {});
    }
    return watchApplication(true, rootPath, config, callback);
  };

  exports.generate = function(options, callback) {
    if (callback == null) {
      callback = (function() {});
    }
    return generateOrDestroy(true, options, callback);
  };

  exports.destroy = function(options, callback) {
    if (callback == null) {
      callback = (function() {});
    }
    return generateOrDestroy(false, options, callback);
  };

}).call(this);
