(function() {
  var argumentum, brunch, commandLineConfig, fs, generatorChoices, helpers, readPackageVersion, sysPath;

  argumentum = require('argumentum');

  fs = require('fs');

  sysPath = require('path');

  brunch = require('./brunch');

  helpers = require('./helpers');

  exports.readPackageVersion = readPackageVersion = function() {
    var content;
    content = fs.readFileSync(sysPath.join(__dirname, '..', 'package.json'));
    return (JSON.parse(content)).version;
  };

  generatorChoices = ['collection', 'model', 'router', 'style', 'template', 'view'];

  commandLineConfig = {
    script: 'brunch',
    commandRequired: true,
    commands: {
      "new": {
        abbr: 'n',
        help: 'Create new brunch project',
        options: {
          rootPath: {
            position: 1,
            help: 'application path',
            metavar: 'APP_PATH',
            required: true,
            full: 'appPath'
          },
          template: {
            abbr: 't',
            help: 'path to application template'
          }
        },
        callback: function(options) {
          return brunch["new"](options);
        }
      },
      build: {
        abbr: 'b',
        help: 'Build a brunch project',
        options: {
          buildPath: {
            abbr: 'o',
            help: 'build path',
            metavar: 'DIRECTORY',
            full: 'output'
          }
        },
        callback: function(options) {
          var config;
          config = helpers.loadConfig('config.coffee');
          if (options.buildPath) config.buildPath = options.buildPath;
          return brunch.build('.', config);
        }
      },
      watch: {
        abbr: 'w',
        help: 'Watch brunch directory and rebuild if something changed',
        options: {
          buildPath: {
            abbr: 'o',
            full: 'output',
            help: 'build path',
            metavar: 'DIRECTORY'
          },
          server: {
            abbr: 's',
            flag: true,
            help: 'run a simple http server that would server `output` dir'
          },
          port: {
            abbr: 'p',
            help: 'if a `server` option was specified, define on which port \
the server would run',
            metavar: 'PORT'
          }
        },
        callback: function(options) {
          var config;
          config = helpers.loadConfig('config.coffee');
          if (config.server == null) config.server = {};
          if (options.server) config.server.run = true;
          if (options.port) config.server.port = options.port;
          if (options.buildPath) config.buildPath = options.buildPath;
          return brunch.watch('.', config);
        }
      },
      generate: {
        abbr: 'g',
        help: 'Generate file(s) from template for current project',
        options: {
          type: {
            position: 1,
            help: 'generator type',
            choices: generatorChoices,
            required: true
          },
          name: {
            position: 2,
            help: 'generator class name / filename',
            required: true
          },
          parentDir: {
            abbr: 'p',
            help: 'path to generated file directory',
            metavar: 'DIRECTORY',
            full: 'path'
          }
        },
        callback: function(options) {
          options.rootPath = '.';
          options.config = helpers.loadConfig('config.coffee');
          return brunch.generate(options);
        }
      },
      destroy: {
        abbr: 'd',
        help: 'Destroy changes made by brunch generate for current project',
        options: {
          type: {
            position: 1,
            help: 'generator type',
            choices: generatorChoices,
            required: true
          },
          name: {
            position: 2,
            help: 'generator class name / filename',
            required: true
          },
          parentDir: {
            abbr: 'p',
            help: 'path to generated file directory',
            metavar: 'DIRECTORY',
            full: 'path'
          }
        },
        callback: function(options) {
          options.rootPath = '.';
          options.config = helpers.loadConfig('config.coffee');
          return brunch.destroy(options);
        }
      }
    },
    options: {
      version: {
        abbr: 'v',
        help: 'display brunch version',
        flag: true,
        callback: readPackageVersion
      }
    }
  };

  exports.run = function() {
    return argumentum.load(commandLineConfig).parse();
  };

}).call(this);
