// Generated by CoffeeScript 1.3.3
(function() {
  'use strict';

  var BrunchWatcher, async, chokidar, fs_utils, getJoinConfig, getPluginIncludes, helpers, isCompilerFor, listToObj, logger, makeUniversalChecker, sysPath, watch,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };

  async = require('async');

  chokidar = require('chokidar');

  sysPath = require('path');

  helpers = require('../helpers');

  logger = require('../logger');

  fs_utils = require('../fs_utils');

  isCompilerFor = function(path, plugin) {
    var pattern;
    pattern = plugin.pattern ? plugin.pattern : plugin.extension ? RegExp("\\." + plugin.extension + "$") : null;
    return (typeof plugin.compile === 'function') && !!(pattern != null ? pattern.test(path) : void 0);
  };

  getPluginIncludes = function(plugins) {
    return plugins.map(function(plugin) {
      return plugin.include;
    }).map(helpers.callFunctionOrPass).filter(function(paths) {
      return paths != null;
    }).reduce((function(acc, elem) {
      return acc.concat(helpers.ensureArray(elem));
    }), []);
  };

  makeUniversalChecker = function(item) {
    switch (toString.call(item)) {
      case '[object RegExp]':
        return function(string) {
          return item.test(string);
        };
      case '[object Function]':
        return item;
      default:
        throw new Error("Config.files item " + item + " is invalid.Use RegExp or Function.");
    }
  };

  listToObj = function(acc, elem) {
    acc[elem[0]] = elem[1];
    return acc;
  };

  getJoinConfig = function(config) {
    var result, types,
      _this = this;
    types = Object.keys(config.files);
    result = types.map(function(type) {
      return config.files[type].joinTo;
    }).map(function(joinTo) {
      var object;
      if (typeof joinTo === 'string') {
        object = {};
        object[joinTo] = /.+/;
        return object;
      } else {
        return joinTo;
      }
    }).map(function(joinTo, index) {
      var makeChecker, subConfig;
      makeChecker = function(generatedFilePath) {
        return [generatedFilePath, makeUniversalChecker(joinTo[generatedFilePath])];
      };
      subConfig = Object.keys(joinTo).map(makeChecker).reduce(listToObj, {});
      return [types[index], subConfig];
    }).reduce(listToObj, {});
    return Object.freeze(result);
  };

  BrunchWatcher = (function() {

    function BrunchWatcher(persistent, options, _onCompile) {
      var params;
      this.persistent = persistent;
      this.options = options;
      this._onCompile = _onCompile;
      this.compile = __bind(this.compile, this);

      this.onCompile = __bind(this.onCompile, this);

      this.removeFromFileList = __bind(this.removeFromFileList, this);

      this.changeFileList = __bind(this.changeFileList, this);

      params = {};
      params.minify = Boolean(this.options.minify);
      params.persistent = persistent;
      if (this.options.publicPath) {
        params.paths = {};
        params.paths["public"] = this.options.publicPath;
      }
      if (persistent) {
        params.server = {};
        if (this.options.server) {
          params.server.run = true;
        }
        if (this.options.port) {
          params.server.port = this.options.port;
        }
      }
      this.configParams = params;
    }

    BrunchWatcher.prototype.clone = function() {
      return new BrunchWatcher(this.persistent, this.options, this.onCompile);
    };

    BrunchWatcher.prototype.initServer = function() {
      if (this.persistent && this.config.server.run) {
        return this.server = helpers.startServer(this.config);
      }
    };

    BrunchWatcher.prototype.initFileList = function() {
      return this.fileList = new fs_utils.FileList(this.config);
    };

    BrunchWatcher.prototype.initPlugins = function(callback) {
      var _this = this;
      return helpers.loadPlugins(this.config, function(error, plugins) {
        if (error != null) {
          return logger.error(error);
        }
        _this.plugins = plugins;
        return callback(error);
      });
    };

    BrunchWatcher.prototype.changeFileList = function(path, isHelper) {
      var compiler, _ref;
      if (isHelper == null) {
        isHelper = false;
      }
      if ((_ref = this.start) == null) {
        this.start = Date.now();
      }
      compiler = this.plugins.filter(isCompilerFor.bind(null, path))[0];
      return this.fileList.emit('change', path, compiler, isHelper);
    };

    BrunchWatcher.prototype.removeFromFileList = function(path) {
      var _ref;
      if ((_ref = this.start) == null) {
        this.start = Date.now();
      }
      return this.fileList.emit('unlink', path);
    };

    BrunchWatcher.prototype.initWatcher = function(callback) {
      var watched,
        _this = this;
      watched = [this.config.paths.app, this.config.paths.test, this.config.paths.config, this.config.paths.packageConfig].concat(this.config.paths.vendor, this.config.paths.assets);
      return async.filter(watched, fs_utils.exists, function(watchedFiles) {
        var ignored;
        ignored = fs_utils.ignored;
        _this.watcher = chokidar.watch(watchedFiles, {
          ignored: ignored,
          persistent: _this.persistent
        }).on('add', function(path) {
          logger.debug('watcher', "File '" + path + "' received event 'add'");
          return _this.changeFileList(path, false);
        }).on('change', function(path) {
          logger.debug('watcher', "File '" + path + "' received event 'change'");
          if (path === _this.config.paths.config) {
            return _this.reload(false);
          } else if (path === _this.config.paths.packageConfig) {
            return _this.reload(true);
          } else {
            return _this.changeFileList(path, false);
          }
        }).on('unlink', function(path) {
          logger.debug('watcher', "File '" + path + "' received event 'unlink'");
          if (path === _this.config.paths.config || path === _this.config.paths.packageConfig) {
            logger.info("Detected removal of config.coffee / package.json. Exiting.");
            return process.exit(0);
          } else {
            return _this.removeFromFileList(path);
          }
        }).on('error', logger.error);
        return callback();
      });
    };

    BrunchWatcher.prototype.onCompile = function(generatedFiles) {
      this._onCompile(generatedFiles);
      this.plugins.filter(function(plugin) {
        return typeof plugin.onCompile === 'function';
      }).forEach(function(plugin) {
        return plugin.onCompile(generatedFiles);
      });
      return this.start = null;
    };

    BrunchWatcher.prototype.compile = function() {
      var _this = this;
      return fs_utils.write(this.fileList, this.config, this.joinConfig, this.plugins, this.start, function(error, result) {
        if (error != null) {
          return logger.error("Write failed: " + error);
        }
        logger.info("compiled in " + (Date.now() - _this.start) + "ms");
        _this.start = null;
        if (!_this.persistent) {
          _this.watcher.close();
        }
        return _this.onCompile(result);
      });
    };

    BrunchWatcher.prototype.watch = function() {
      var _this = this;
      return helpers.loadPackages(this.options, function(error, packages) {
        _this.config = helpers.loadConfig(_this.options.configPath, _this.configParams);
        _this.joinConfig = getJoinConfig(_this.config);
        _this.plugins = helpers.getPlugins(packages, _this.config);
        _this.initServer();
        _this.initFileList();
        getPluginIncludes(_this.plugins).forEach(function(path) {
          return _this.changeFileList(path, true);
        });
        return _this.initWatcher(function() {
          return _this.fileList.on('ready', _this.compile);
        });
      });
    };

    BrunchWatcher.prototype.close = function() {
      var _ref;
      if ((_ref = this.server) != null) {
        _ref.close();
      }
      return this.watcher.close();
    };

    BrunchWatcher.prototype.reload = function(reInstall) {
      var reWatch,
        _this = this;
      if (reInstall == null) {
        reInstall = false;
      }
      reWatch = function() {
        _this.close();
        return _this.clone().watch();
      };
      if (reInstall) {
        return helpers.install(this.config.paths.root, reWatch);
      } else {
        return reWatch();
      }
    };

    return BrunchWatcher;

  })();

  module.exports = watch = function(persistent, options, callback) {
    if (callback == null) {
      callback = (function() {});
    }
    return new BrunchWatcher(persistent, options, callback).watch();
  };

}).call(this);
