(function() {
  var EventEmitter, fs, logger, sysPath,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = Object.prototype.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor; child.__super__ = parent.prototype; return child; },
    __indexOf = Array.prototype.indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

  EventEmitter = require('events').EventEmitter;

  fs = require('fs');

  sysPath = require('path');

  logger = require('../logger');

  exports.FileWatcher = (function(_super) {

    __extends(FileWatcher, _super);

    FileWatcher.prototype.invalid = /^(\.|#)/;

    function FileWatcher() {
      this._handle = __bind(this._handle, this);      this.watched = {};
    }

    FileWatcher.prototype._getWatchedDir = function(directory) {
      var _base, _ref;
      return (_ref = (_base = this.watched)[directory]) != null ? _ref : _base[directory] = [];
    };

    FileWatcher.prototype._watch = function(item, callback) {
      var basename, parent,
        _this = this;
      parent = this._getWatchedDir(sysPath.dirname(item));
      basename = sysPath.basename(item);
      if (__indexOf.call(parent, basename) >= 0) return;
      parent.push(basename);
      return fs.watchFile(item, {
        persistent: true,
        interval: 100
      }, function(curr, prev) {
        if (curr.mtime.getTime() !== prev.mtime.getTime()) {
          return typeof callback === "function" ? callback(item) : void 0;
        }
      });
    };

    FileWatcher.prototype._handleFile = function(file) {
      var emit,
        _this = this;
      emit = function(file) {
        return _this.emit('change', file);
      };
      emit(file);
      return this._watch(file, emit);
    };

    FileWatcher.prototype._handleDir = function(directory) {
      var read,
        _this = this;
      read = function(directory) {
        return fs.readdir(directory, function(error, current) {
          var previous;
          if (error != null) return logger.error(error);
          if (!current) return;
          previous = _this._getWatchedDir(directory);
          previous.filter(function(file) {
            return __indexOf.call(current, file) < 0;
          }).forEach(function(file) {
            return _this.emit('remove', sysPath.join(directory, file));
          });
          return current.filter(function(file) {
            return __indexOf.call(previous, file) < 0;
          }).forEach(function(file) {
            return _this._handle(sysPath.join(directory, file));
          });
        });
      };
      read(directory);
      return this._watch(directory, read);
    };

    FileWatcher.prototype._handle = function(file) {
      var _this = this;
      if (this.invalid.test(sysPath.basename(file))) return;
      return fs.realpath(file, function(error, path) {
        if (error != null) return logger.error(error);
        return fs.stat(file, function(error, stats) {
          if (error != null) return logger.error(error);
          if (stats.isFile()) _this._handleFile(file);
          if (stats.isDirectory()) return _this._handleDir(file);
        });
      });
    };

    FileWatcher.prototype.add = function(files) {
      files.forEach(this._handle);
      return this;
    };

    FileWatcher.prototype.on = function() {
      FileWatcher.__super__.on.apply(this, arguments);
      return this;
    };

    FileWatcher.prototype.close = function() {
      var _this = this;
      (Object.keys(this.watched)).forEach(function(directory) {
        return _this.watched[directory].forEach(function(file) {
          return fs.unwatchFile(sysPath.join(directory, file));
        });
      });
      this.watched = {};
      return this;
    };

    return FileWatcher;

  })(EventEmitter);

}).call(this);
