(function() {
  var EventEmitter, GeneratedFile, async, helpers, logger, sysPath,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = Object.prototype.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor; child.__super__ = parent.prototype; return child; };

  async = require('async');

  EventEmitter = require('events').EventEmitter;

  sysPath = require('path');

  helpers = require('../helpers');

  GeneratedFile = require('./generated_file').GeneratedFile;

  logger = require('../logger');

  exports.FileWriter = (function(_super) {

    __extends(FileWriter, _super);

    function FileWriter(config, plugins) {
      this.config = config;
      this.plugins = plugins;
      this.write = __bind(this.write, this);
      this.destFiles = [];
      this._initFilesConfig(this.config.files);
    }

    FileWriter.prototype._initFilesConfig = function(filesConfig) {
      var config,
        _this = this;
      config = filesConfig;
      Object.keys(config).forEach(function(type) {
        var data, object;
        data = config[type];
        if (typeof data.joinTo === 'string') {
          object = {};
          object[data.joinTo] = /.+/;
          data.joinTo = object;
        }
        return Object.keys(data.joinTo).forEach(function(destinationPath) {
          var regExpOrFunction;
          regExpOrFunction = data.joinTo[destinationPath];
          return data.joinTo[destinationPath] = regExpOrFunction instanceof RegExp ? function(string) {
            return regExpOrFunction.test(string);
          } : regExpOrFunction;
        });
      });
      return config;
    };

    FileWriter.prototype._getDestinationPathes = function(file) {
      var data, pathes;
      data = this.config.files[helpers.pluralize(file.type)];
      pathes = (Object.keys(data.joinTo)).filter(function(destinationPath) {
        var checkingFunction;
        checkingFunction = data.joinTo[destinationPath];
        return checkingFunction(file.path);
      });
      if (pathes.length > 0) {
        return pathes;
      } else {
        return null;
      }
    };

    FileWriter.prototype._getFiles = function(fileList, minifiers) {
      var map,
        _this = this;
      map = {};
      fileList.files.forEach(function(file) {
        var pathes;
        pathes = _this._getDestinationPathes(file);
        if (pathes == null) return;
        return pathes.forEach(function(path) {
          if (map[path] == null) map[path] = [];
          return map[path].push(file);
        });
      });
      return (Object.keys(map)).map(function(generatedFilePath) {
        var file, fullPath, sourceFiles;
        sourceFiles = map[generatedFilePath];
        fullPath = sysPath.join(_this.config.buildPath, generatedFilePath);
        file = new GeneratedFile(fullPath, sourceFiles, _this.config);
        minifiers.filter(function(minifier) {
          return minifier.type === file.type;
        }).forEach(function(minifier) {
          return file.minifier = minifier;
        });
        return file;
      });
    };

    FileWriter.prototype.write = function(fileList) {
      var files, write,
        _this = this;
      files = this._getFiles(fileList, this.plugins.filter(function(plugin) {
        return !!plugin.minify;
      }));
      write = function(file, callback) {
        return file.write(callback);
      };
      return async.forEach(files, write, function(error, results) {
        if (error != null) return logger.error("write error. " + error);
        return _this.emit('write', results);
      });
    };

    return FileWriter;

  })(EventEmitter);

}).call(this);
